@@ -91,7 +91,7 @@ typedef struct pooldesc_s
 static pooldesc_t mainpool;
 
 
-/* A couple of flags with some being set early.  */
+/* A couple of flags whith some beeing set early. */
 static int disable_secmem;
 static int show_warning;
 static int not_locked;
@@ -99,8 +99,6 @@ static int no_warning;
 static int suspend_warning;
 static int no_mlock;
 static int no_priv_drop;
-static unsigned int auto_expand;
-
 
 /* Lock protecting accesses to the memory pools.  */
 GPGRT_LOCK_DEFINE (secmem_lock);
@@ -409,7 +407,7 @@ init_pool (pooldesc_t *pool, size_t n)
 # else
     pgsize_val = -1;
 # endif
-    pgsize = (pgsize_val > 0)? pgsize_val:DEFAULT_PAGE_SIZE;
+    pgsize = (pgsize_val != -1 && pgsize_val > 0)? pgsize_val:DEFAULT_PAGE_SIZE;
 
     pool->size = (pool->size + pgsize - 1) & ~(pgsize - 1);
 # ifdef MAP_ANONYMOUS
@@ -456,28 +454,10 @@ init_pool (pooldesc_t *pool, size_t n)
 
   /* Initialize first memory block.  */
   mb = (memblock_t *) pool->mem;
-  mb->size = pool->size - BLOCK_HEAD_SIZE;
+  mb->size = pool->size;
   mb->flags = 0;
 }
 
-
-/* Enable overflow pool allocation in all cases.  CHUNKSIZE is a hint
- * on how large to allocate overflow pools.  */
-void
-_gcry_secmem_set_auto_expand (unsigned int chunksize)
-{
-  /* Round up to a multiple of the STANDARD_POOL_SIZE.  */
-  chunksize = ((chunksize + (2*STANDARD_POOL_SIZE) - 1)
-               / STANDARD_POOL_SIZE ) * STANDARD_POOL_SIZE;
-  if (chunksize < STANDARD_POOL_SIZE) /* In case of overflow.  */
-    chunksize = STANDARD_POOL_SIZE;
-
-  SECMEM_LOCK;
-  auto_expand = chunksize;
-  SECMEM_UNLOCK;
-}
-
-
 void
 _gcry_secmem_set_flags (unsigned flags)
 {
@@ -630,21 +610,21 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
   mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
   if (mb)
     {
-      stats_update (pool, mb->size, 0);
+      stats_update (pool, size, 0);
       return &mb->aligned.c;
     }
 
   /* If we are called from xmalloc style function resort to the
    * overflow pools to return memory.  We don't do this in FIPS mode,
    * though. */
-  if ((xhint || auto_expand) && !fips_mode ())
+  if (xhint && !fips_mode ())
     {
       for (pool = pool->next; pool; pool = pool->next)
         {
           mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
           if (mb)
             {
-              stats_update (pool, mb->size, 0);
+              stats_update (pool, size, 0);
               return &mb->aligned.c;
             }
         }
@@ -655,16 +635,13 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
       pool = calloc (1, sizeof *pool);
       if (!pool)
         return NULL;  /* Not enough memory for a new pool descriptor.  */
-      pool->size = auto_expand? auto_expand : STANDARD_POOL_SIZE;
+      pool->size = STANDARD_POOL_SIZE;
       pool->mem = malloc (pool->size);
       if (!pool->mem)
-        {
-          free (pool);
-          return NULL; /* Not enough memory available for a new pool.  */
-        }
+        return NULL; /* Not enough memory available for a new pool.  */
       /* Initialize first memory block.  */
       mb = (memblock_t *) pool->mem;
-      mb->size = pool->size - BLOCK_HEAD_SIZE;
+      mb->size = pool->size;
       mb->flags = 0;
 
       pool->okay = 1;
@@ -683,7 +660,7 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
       mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
       if (mb)
         {
-          stats_update (pool, mb->size, 0);
+          stats_update (pool, size, 0);
           return &mb->aligned.c;
         }
     }
@@ -864,8 +841,6 @@ _gcry_secmem_term ()
 }
 
 
-/* Print stats of the secmem allocator.  With EXTENDED passwed as true
- * a detiled listing is returned (used for testing).  */
 void
 _gcry_secmem_dump_stats (int extended)
 {
@@ -897,5 +872,6 @@ _gcry_secmem_dump_stats (int extended)
                       mb->size);
         }
     }
+
   SECMEM_UNLOCK;
 }
