@@ -39,7 +39,6 @@
 #endif
 
 #define PGM "bench-slope"
-#include "t-common.h"
 
 static int verbose;
 static int csv_mode;
@@ -50,9 +49,6 @@ static int num_measurement_repetitions;
    results.  */
 static double cpu_ghz = -1;
 
-/* Attempt to autodetect CPU Ghz. */
-static int auto_ghz;
-
 /* Whether we are running as part of the regression test suite.  */
 static int in_regression_test;
 
@@ -64,13 +60,6 @@ static char *current_algo_name;
 static char *current_mode_name;
 
 
-/* Currently used CPU Ghz (either user input or auto-detected. */
-static double bench_ghz;
-
-/* Current accuracy of auto-detected CPU Ghz. */
-static double bench_ghz_diff;
-
-
 /*************************************** Default parameters for measurements. */
 
 /* Start at small buffer size, to get reasonable timer calibration for fast
@@ -89,9 +78,6 @@ static double bench_ghz_diff;
  * measurements is selected as data point further analysis. */
 #define NUM_MEASUREMENT_REPETITIONS	64
 
-/* Target accuracy for auto-detected CPU Ghz. */
-#define AUTO_GHZ_TARGET_DIFF		(5e-5)
-
 /**************************************************** High-resolution timers. */
 
 /* This benchmarking module needs needs high resolution timer.  */
@@ -233,7 +219,6 @@ struct bench_obj
   unsigned int step_size;
 
   void *priv;
-  void *hd;
 };
 
 typedef int (*const bench_initialize_t) (struct bench_obj * obj);
@@ -397,7 +382,7 @@ adjust_loop_iterations_to_timer_accuracy (struct bench_obj *obj, void *buffer,
 
 /* Benchmark and return linear regression slope in nanoseconds per byte.  */
 double
-slope_benchmark (struct bench_obj *obj)
+do_slope_benchmark (struct bench_obj *obj)
 {
   unsigned int num_measurements;
   double *measurements = NULL;
@@ -478,176 +463,6 @@ err_free:
   return -1;
 }
 
-/********************************************* CPU frequency auto-detection. */
-
-static int
-auto_ghz_init (struct bench_obj *obj)
-{
-  obj->min_bufsize = 16;
-  obj->max_bufsize = 64 + obj->min_bufsize;
-  obj->step_size = 8;
-  obj->num_measure_repetitions = 16;
-
-  return 0;
-}
-
-static void
-auto_ghz_free (struct bench_obj *obj)
-{
-  (void)obj;
-}
-
-static void
-auto_ghz_bench (struct bench_obj *obj, void *buf, size_t buflen)
-{
-  (void)obj;
-  (void)buf;
-
-  buflen *= 1024;
-
-  /* Turbo frequency detection benchmark. Without CPU turbo-boost, this
-   * function will give cycles/iteration result 1024.0 on high-end CPUs.
-   * With turbo, result will be less and can be used detect turbo-clock. */
-
-#ifdef HAVE_GCC_ASM_VOLATILE_MEMORY
-  /* Auto-ghz operation takes two CPU cycles to perform. Memory barriers
-   * are used to prevent compiler from optimizing this loop away. */
-  #define AUTO_GHZ_OPERATION \
-	asm volatile ("":"+r"(buflen)::"memory"); \
-	buflen ^= 1; \
-	asm volatile ("":"+r"(buflen)::"memory"); \
-	buflen -= 2
-#else
-  /* TODO: Needs alternative way of preventing compiler optimizations.
-   *       Mix of XOR and subtraction appears to do the trick for now. */
-  #define AUTO_GHZ_OPERATION \
-	buflen ^= 1; \
-	buflen -= 2
-#endif
-
-#define AUTO_GHZ_OPERATION_2 \
-	AUTO_GHZ_OPERATION; \
-	AUTO_GHZ_OPERATION
-
-#define AUTO_GHZ_OPERATION_4 \
-	AUTO_GHZ_OPERATION_2; \
-	AUTO_GHZ_OPERATION_2
-
-#define AUTO_GHZ_OPERATION_8 \
-	AUTO_GHZ_OPERATION_4; \
-	AUTO_GHZ_OPERATION_4
-
-#define AUTO_GHZ_OPERATION_16 \
-	AUTO_GHZ_OPERATION_8; \
-	AUTO_GHZ_OPERATION_8
-
-#define AUTO_GHZ_OPERATION_32 \
-	AUTO_GHZ_OPERATION_16; \
-	AUTO_GHZ_OPERATION_16
-
-#define AUTO_GHZ_OPERATION_64 \
-	AUTO_GHZ_OPERATION_32; \
-	AUTO_GHZ_OPERATION_32
-
-#define AUTO_GHZ_OPERATION_128 \
-	AUTO_GHZ_OPERATION_64; \
-	AUTO_GHZ_OPERATION_64
-
-  do
-    {
-      /* 1024 auto-ghz operations per loop, total 2048 instructions. */
-      AUTO_GHZ_OPERATION_128; AUTO_GHZ_OPERATION_128;
-      AUTO_GHZ_OPERATION_128; AUTO_GHZ_OPERATION_128;
-      AUTO_GHZ_OPERATION_128; AUTO_GHZ_OPERATION_128;
-      AUTO_GHZ_OPERATION_128; AUTO_GHZ_OPERATION_128;
-    }
-  while (buflen);
-}
-
-static struct bench_ops auto_ghz_detect_ops = {
-  &auto_ghz_init,
-  &auto_ghz_free,
-  &auto_ghz_bench
-};
-
-
-double
-get_auto_ghz (void)
-{
-  struct bench_obj obj = { 0 };
-  double nsecs_per_iteration;
-  double cycles_per_iteration;
-
-  obj.ops = &auto_ghz_detect_ops;
-
-  nsecs_per_iteration = slope_benchmark (&obj);
-
-  cycles_per_iteration = nsecs_per_iteration * cpu_ghz;
-
-  /* Adjust CPU Ghz so that cycles per iteration would give '1024.0'. */
-
-  return cpu_ghz * 1024 / cycles_per_iteration;
-}
-
-
-double
-do_slope_benchmark (struct bench_obj *obj)
-{
-  double ret;
-
-  if (!auto_ghz)
-    {
-      /* Perform measurement without autodetection of CPU frequency. */
-
-      ret = slope_benchmark (obj);
-
-      bench_ghz = cpu_ghz;
-      bench_ghz_diff = 0;
-    }
-  else
-    {
-      double target_diff = AUTO_GHZ_TARGET_DIFF;
-      double cpu_auto_ghz_before;
-      double cpu_auto_ghz_after;
-      double nsecs_per_iteration;
-      double diff;
-      unsigned int try_count = 0;
-
-      /* Perform measurement with CPU frequency autodetection. */
-
-      do
-        {
-          /* Repeat measurement until CPU turbo frequency has stabilized. */
-
-	  if (try_count++ > 4)
-	    {
-	      /* Too much frequency instability on the system, relax target
-	       * accuracy. */
-
-	      try_count = 0;
-	      target_diff *= 2;
-	    }
-
-          cpu_auto_ghz_before = get_auto_ghz ();
-
-          nsecs_per_iteration = slope_benchmark (obj);
-
-          cpu_auto_ghz_after = get_auto_ghz ();
-
-          diff = 1.0 - (cpu_auto_ghz_before / cpu_auto_ghz_after);
-          diff = diff < 0 ? -diff : diff;
-        }
-      while (diff > target_diff);
-
-      ret = nsecs_per_iteration;
-
-      bench_ghz = (cpu_auto_ghz_before + cpu_auto_ghz_after) / 2;
-      bench_ghz_diff = diff;
-    }
-
-  return ret;
-}
-
 
 /********************************************************** Printing results. */
 
@@ -660,10 +475,8 @@ double_to_str (char *out, size_t outlen, double value)
     fmt = "%.3f";
   else if (value < 100.0)
     fmt = "%.2f";
-  else if (value < 1000.0)
-    fmt = "%.1f";
   else
-    fmt = "%.0f";
+    fmt = "%.1f";
 
   snprintf (out, outlen, fmt, value);
 }
@@ -675,26 +488,16 @@ bench_print_result_csv (double nsecs_per_byte)
   char nsecpbyte_buf[16];
   char mbpsec_buf[16];
   char cpbyte_buf[16];
-  char mhz_buf[16];
-  char mhz_diff_buf[32];
 
-  strcpy (mhz_diff_buf, "");
   *cpbyte_buf = 0;
-  *mhz_buf = 0;
 
   double_to_str (nsecpbyte_buf, sizeof (nsecpbyte_buf), nsecs_per_byte);
 
   /* If user didn't provide CPU speed, we cannot show cycles/byte results.  */
-  if (bench_ghz > 0.0)
+  if (cpu_ghz > 0.0)
     {
-      cycles_per_byte = nsecs_per_byte * bench_ghz;
+      cycles_per_byte = nsecs_per_byte * cpu_ghz;
       double_to_str (cpbyte_buf, sizeof (cpbyte_buf), cycles_per_byte);
-      double_to_str (mhz_buf, sizeof (mhz_buf), bench_ghz * 1000);
-      if (auto_ghz && bench_ghz_diff * 1000 >= 1)
-	{
-	  snprintf(mhz_diff_buf, sizeof(mhz_diff_buf), ",%.0f,Mhz-diff",
-		   bench_ghz_diff * 1000);
-	}
     }
 
   mbytes_per_sec =
@@ -702,28 +505,14 @@ bench_print_result_csv (double nsecs_per_byte)
   double_to_str (mbpsec_buf, sizeof (mbpsec_buf), mbytes_per_sec);
 
   /* We print two empty fields to allow for future enhancements.  */
-  if (auto_ghz)
-    {
-      printf ("%s,%s,%s,,,%s,ns/B,%s,MiB/s,%s,c/B,%s,Mhz%s\n",
-              current_section_name,
-              current_algo_name? current_algo_name : "",
-              current_mode_name? current_mode_name : "",
-              nsecpbyte_buf,
-              mbpsec_buf,
-              cpbyte_buf,
-              mhz_buf,
-              mhz_diff_buf);
-    }
-  else
-    {
-      printf ("%s,%s,%s,,,%s,ns/B,%s,MiB/s,%s,c/B\n",
-              current_section_name,
-              current_algo_name? current_algo_name : "",
-              current_mode_name? current_mode_name : "",
-              nsecpbyte_buf,
-              mbpsec_buf,
-              cpbyte_buf);
-    }
+  printf ("%s,%s,%s,,,%s,ns/B,%s,MiB/s,%s,c/B\n",
+          current_section_name,
+          current_algo_name? current_algo_name : "",
+          current_mode_name? current_mode_name : "",
+          nsecpbyte_buf,
+          mbpsec_buf,
+          cpbyte_buf);
+
 }
 
 static void
@@ -733,45 +522,24 @@ bench_print_result_std (double nsecs_per_byte)
   char nsecpbyte_buf[16];
   char mbpsec_buf[16];
   char cpbyte_buf[16];
-  char mhz_buf[16];
-  char mhz_diff_buf[32];
-
-  strcpy (mhz_diff_buf, "");
 
   double_to_str (nsecpbyte_buf, sizeof (nsecpbyte_buf), nsecs_per_byte);
 
   /* If user didn't provide CPU speed, we cannot show cycles/byte results.  */
-  if (bench_ghz > 0.0)
+  if (cpu_ghz > 0.0)
     {
-      cycles_per_byte = nsecs_per_byte * bench_ghz;
+      cycles_per_byte = nsecs_per_byte * cpu_ghz;
       double_to_str (cpbyte_buf, sizeof (cpbyte_buf), cycles_per_byte);
-      double_to_str (mhz_buf, sizeof (mhz_buf), bench_ghz * 1000);
-      if (auto_ghz && bench_ghz_diff * 1000 >= 0.5)
-	{
-	  snprintf(mhz_diff_buf, sizeof(mhz_diff_buf), "±%.0f",
-		   bench_ghz_diff * 1000);
-	}
     }
   else
-    {
-      strcpy (cpbyte_buf, "-");
-      strcpy (mhz_buf, "-");
-    }
+    strcpy (cpbyte_buf, "-");
 
   mbytes_per_sec =
     (1000.0 * 1000.0 * 1000.0) / (nsecs_per_byte * 1024 * 1024);
   double_to_str (mbpsec_buf, sizeof (mbpsec_buf), mbytes_per_sec);
 
-  if (auto_ghz)
-    {
-      printf ("%9s ns/B %9s MiB/s %9s c/B %9s%s\n",
-              nsecpbyte_buf, mbpsec_buf, cpbyte_buf, mhz_buf, mhz_diff_buf);
-    }
-  else
-    {
-      printf ("%9s ns/B %9s MiB/s %9s c/B\n",
-              nsecpbyte_buf, mbpsec_buf, cpbyte_buf);
-    }
+  printf ("%9s ns/B %9s MiB/s %9s c/B\n",
+          nsecpbyte_buf, mbpsec_buf, cpbyte_buf);
 }
 
 static void
@@ -809,13 +577,8 @@ bench_print_header (int algo_width, const char *algo_name)
         printf (" %-*s | ", -algo_width, algo_name);
       else
         printf (" %-*s | ", algo_width, algo_name);
-
-      if (auto_ghz)
-        printf ("%14s %15s %13s %9s\n", "nanosecs/byte", "mebibytes/sec",
-                "cycles/byte", "auto Mhz");
-      else
-        printf ("%14s %15s %13s\n", "nanosecs/byte", "mebibytes/sec",
-                "cycles/byte");
+      printf ("%14s %15s %13s\n", "nanosecs/byte", "mebibytes/sec",
+              "cycles/byte");
     }
 }
 
@@ -920,7 +683,7 @@ bench_encrypt_init (struct bench_obj *obj)
       exit (1);
     }
 
-  obj->hd = hd;
+  obj->priv = hd;
 
   return 0;
 }
@@ -928,7 +691,7 @@ bench_encrypt_init (struct bench_obj *obj)
 static void
 bench_encrypt_free (struct bench_obj *obj)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
 
   gcry_cipher_close (hd);
 }
@@ -936,12 +699,10 @@ bench_encrypt_free (struct bench_obj *obj)
 static void
 bench_encrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
 
-  err = gcry_cipher_reset (hd);
-  if (!err)
-    err = gcry_cipher_encrypt (hd, buf, buflen, buf, buflen);
+  err = gcry_cipher_encrypt (hd, buf, buflen, buf, buflen);
   if (err)
     {
       fprintf (stderr, PGM ": gcry_cipher_encrypt failed: %s\n",
@@ -954,12 +715,10 @@ bench_encrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 static void
 bench_decrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
 
-  err = gcry_cipher_reset (hd);
-  if (!err)
-    err = gcry_cipher_decrypt (hd, buf, buflen, buf, buflen);
+  err = gcry_cipher_decrypt (hd, buf, buflen, buf, buflen);
   if (err)
     {
       fprintf (stderr, PGM ": gcry_cipher_encrypt failed: %s\n",
@@ -982,75 +741,10 @@ static struct bench_ops decrypt_ops = {
 };
 
 
-static int
-bench_xts_encrypt_init (struct bench_obj *obj)
-{
-  struct bench_cipher_mode *mode = obj->priv;
-  gcry_cipher_hd_t hd;
-  int err, keylen;
-
-  obj->min_bufsize = BUF_START_SIZE;
-  obj->max_bufsize = BUF_END_SIZE;
-  obj->step_size = BUF_STEP_SIZE;
-  obj->num_measure_repetitions = num_measurement_repetitions;
-
-  err = gcry_cipher_open (&hd, mode->algo, mode->mode, 0);
-  if (err)
-    {
-      fprintf (stderr, PGM ": error opening cipher `%s'\n",
-	       gcry_cipher_algo_name (mode->algo));
-      exit (1);
-    }
-
-  /* Double key-length for XTS. */
-  keylen = gcry_cipher_get_algo_keylen (mode->algo) * 2;
-  if (keylen)
-    {
-      char key[keylen];
-      int i;
-
-      for (i = 0; i < keylen; i++)
-	key[i] = 0x33 ^ (11 - i);
-
-      err = gcry_cipher_setkey (hd, key, keylen);
-      if (err)
-	{
-	  fprintf (stderr, PGM ": gcry_cipher_setkey failed: %s\n",
-		   gpg_strerror (err));
-	  gcry_cipher_close (hd);
-	  exit (1);
-	}
-    }
-  else
-    {
-      fprintf (stderr, PGM ": failed to get key length for algorithm `%s'\n",
-	       gcry_cipher_algo_name (mode->algo));
-      gcry_cipher_close (hd);
-      exit (1);
-    }
-
-  obj->hd = hd;
-
-  return 0;
-}
-
-static struct bench_ops xts_encrypt_ops = {
-  &bench_xts_encrypt_init,
-  &bench_encrypt_free,
-  &bench_encrypt_do_bench
-};
-
-static struct bench_ops xts_decrypt_ops = {
-  &bench_xts_encrypt_init,
-  &bench_encrypt_free,
-  &bench_decrypt_do_bench
-};
-
-
 static void
 bench_ccm_encrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[8];
   char nonce[11] = { 0x80, 0x01, };
@@ -1094,7 +788,7 @@ bench_ccm_encrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 static void
 bench_ccm_decrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[8] = { 0, };
   char nonce[11] = { 0x80, 0x01, };
@@ -1141,7 +835,7 @@ static void
 bench_ccm_authenticate_do_bench (struct bench_obj *obj, void *buf,
 				 size_t buflen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[8] = { 0, };
   char nonce[11] = { 0x80, 0x01, };
@@ -1215,7 +909,7 @@ static void
 bench_aead_encrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen,
 			     const char *nonce, size_t noncelen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[16];
 
@@ -1245,7 +939,7 @@ static void
 bench_aead_decrypt_do_bench (struct bench_obj *obj, void *buf, size_t buflen,
 			     const char *nonce, size_t noncelen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[16] = { 0, };
 
@@ -1278,7 +972,7 @@ bench_aead_authenticate_do_bench (struct bench_obj *obj, void *buf,
 				  size_t buflen, const char *nonce,
 				  size_t noncelen)
 {
-  gcry_cipher_hd_t hd = obj->hd;
+  gcry_cipher_hd_t hd = obj->priv;
   int err;
   char tag[16] = { 0, };
   char data = 0xff;
@@ -1416,53 +1110,6 @@ static struct bench_ops ocb_authenticate_ops = {
   &bench_ocb_authenticate_do_bench
 };
 
-static void
-bench_eax_encrypt_do_bench (struct bench_obj *obj, void *buf,
-			    size_t buflen)
-{
-  char nonce[16] = { 0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce,
-                     0xdb, 0xad, 0xde, 0xca, 0xf8, 0x88,
-                     0x00, 0x00, 0x01, 0x00 };
-  bench_aead_encrypt_do_bench (obj, buf, buflen, nonce, sizeof(nonce));
-}
-
-static void
-bench_eax_decrypt_do_bench (struct bench_obj *obj, void *buf,
-			    size_t buflen)
-{
-  char nonce[16] = { 0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce,
-                     0xdb, 0xad, 0xde, 0xca, 0xf8, 0x88,
-                     0x00, 0x00, 0x01, 0x00 };
-  bench_aead_decrypt_do_bench (obj, buf, buflen, nonce, sizeof(nonce));
-}
-
-static void
-bench_eax_authenticate_do_bench (struct bench_obj *obj, void *buf,
-				 size_t buflen)
-{
-  char nonce[16] = { 0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce,
-                     0xdb, 0xad, 0xde, 0xca, 0xf8, 0x88,
-                     0x00, 0x00, 0x01, 0x00 };
-  bench_aead_authenticate_do_bench (obj, buf, buflen, nonce, sizeof(nonce));
-}
-
-static struct bench_ops eax_encrypt_ops = {
-  &bench_encrypt_init,
-  &bench_encrypt_free,
-  &bench_eax_encrypt_do_bench
-};
-
-static struct bench_ops eax_decrypt_ops = {
-  &bench_encrypt_init,
-  &bench_encrypt_free,
-  &bench_eax_decrypt_do_bench
-};
-
-static struct bench_ops eax_authenticate_ops = {
-  &bench_encrypt_init,
-  &bench_encrypt_free,
-  &bench_eax_authenticate_do_bench
-};
 
 static void
 bench_poly1305_encrypt_do_bench (struct bench_obj *obj, void *buf,
@@ -1518,14 +1165,9 @@ static struct bench_cipher_mode cipher_modes[] = {
   {GCRY_CIPHER_MODE_OFB, "OFB dec", &decrypt_ops},
   {GCRY_CIPHER_MODE_CTR, "CTR enc", &encrypt_ops},
   {GCRY_CIPHER_MODE_CTR, "CTR dec", &decrypt_ops},
-  {GCRY_CIPHER_MODE_XTS, "XTS enc", &xts_encrypt_ops},
-  {GCRY_CIPHER_MODE_XTS, "XTS dec", &xts_decrypt_ops},
   {GCRY_CIPHER_MODE_CCM, "CCM enc", &ccm_encrypt_ops},
   {GCRY_CIPHER_MODE_CCM, "CCM dec", &ccm_decrypt_ops},
   {GCRY_CIPHER_MODE_CCM, "CCM auth", &ccm_authenticate_ops},
-  {GCRY_CIPHER_MODE_EAX, "EAX enc",  &eax_encrypt_ops},
-  {GCRY_CIPHER_MODE_EAX, "EAX dec",  &eax_decrypt_ops},
-  {GCRY_CIPHER_MODE_EAX, "EAX auth", &eax_authenticate_ops},
   {GCRY_CIPHER_MODE_GCM, "GCM enc", &gcm_encrypt_ops},
   {GCRY_CIPHER_MODE_GCM, "GCM dec", &gcm_decrypt_ops},
   {GCRY_CIPHER_MODE_GCM, "GCM auth", &gcm_authenticate_ops},
@@ -1576,12 +1218,8 @@ cipher_bench_one (int algo, struct bench_cipher_mode *pmode)
   if (mode.mode == GCRY_CIPHER_MODE_GCM && blklen != GCRY_GCM_BLOCK_LEN)
     return;
 
-  /* XTS has restrictions for block-size */
-  if (mode.mode == GCRY_CIPHER_MODE_XTS && blklen != GCRY_XTS_BLOCK_LEN)
-    return;
-
-  /* Our OCB implementation has restrictions for block-size.  */
-  if (mode.mode == GCRY_CIPHER_MODE_OCB && blklen != GCRY_OCB_BLOCK_LEN)
+  /* Our OCB implementaion has restrictions for block-size.  */
+  if (mode.mode == GCRY_CIPHER_MODE_OCB && blklen != 16)
     return;
 
   bench_print_mode (14, mode.name);
@@ -1668,7 +1306,7 @@ bench_hash_init (struct bench_obj *obj)
       exit (1);
     }
 
-  obj->hd = hd;
+  obj->priv = hd;
 
   return 0;
 }
@@ -1676,7 +1314,7 @@ bench_hash_init (struct bench_obj *obj)
 static void
 bench_hash_free (struct bench_obj *obj)
 {
-  gcry_md_hd_t hd = obj->hd;
+  gcry_md_hd_t hd = obj->priv;
 
   gcry_md_close (hd);
 }
@@ -1684,7 +1322,7 @@ bench_hash_free (struct bench_obj *obj)
 static void
 bench_hash_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_md_hd_t hd = obj->hd;
+  gcry_md_hd_t hd = obj->priv;
 
   gcry_md_reset (hd);
   gcry_md_write (hd, buf, buflen);
@@ -1830,7 +1468,7 @@ bench_mac_init (struct bench_obj *obj)
       break;
     }
 
-  obj->hd = hd;
+  obj->priv = hd;
 
   free (key);
   return 0;
@@ -1839,7 +1477,7 @@ bench_mac_init (struct bench_obj *obj)
 static void
 bench_mac_free (struct bench_obj *obj)
 {
-  gcry_mac_hd_t hd = obj->hd;
+  gcry_mac_hd_t hd = obj->priv;
 
   gcry_mac_close (hd);
 }
@@ -1847,7 +1485,7 @@ bench_mac_free (struct bench_obj *obj)
 static void
 bench_mac_do_bench (struct bench_obj *obj, void *buf, size_t buflen)
 {
-  gcry_mac_hd_t hd = obj->hd;
+  gcry_mac_hd_t hd = obj->priv;
   size_t bs;
   char b;
 
@@ -1995,7 +1633,6 @@ kdf_bench_one (int algo, int subalgo)
   char algo_name[32];
   char nsecpiter_buf[16];
   char cpiter_buf[16];
-  char mhz_buf[16];
 
   mode.algo = algo;
   mode.subalgo = subalgo;
@@ -2032,42 +1669,28 @@ kdf_bench_one (int algo, int subalgo)
   nsecs_per_iteration = do_slope_benchmark (&obj);
 
   strcpy(cpiter_buf, csv_mode ? "" : "-");
-  strcpy(mhz_buf, csv_mode ? "" : "-");
 
   double_to_str (nsecpiter_buf, sizeof (nsecpiter_buf), nsecs_per_iteration);
 
   /* If user didn't provide CPU speed, we cannot show cycles/iter results.  */
-  if (bench_ghz > 0.0)
+  if (cpu_ghz > 0.0)
     {
-      cycles_per_iteration = nsecs_per_iteration * bench_ghz;
+      cycles_per_iteration = nsecs_per_iteration * cpu_ghz;
       double_to_str (cpiter_buf, sizeof (cpiter_buf), cycles_per_iteration);
-      double_to_str (mhz_buf, sizeof (mhz_buf), bench_ghz * 1000);
     }
 
   if (csv_mode)
     {
-      if (auto_ghz)
-        printf ("%s,%s,%s,,,,,,,,,%s,ns/iter,%s,c/iter,%s,Mhz\n",
-                current_section_name,
-                current_algo_name ? current_algo_name : "",
-                current_mode_name ? current_mode_name : "",
-                nsecpiter_buf,
-                cpiter_buf,
-                mhz_buf);
-      else
-        printf ("%s,%s,%s,,,,,,,,,%s,ns/iter,%s,c/iter\n",
-                current_section_name,
-                current_algo_name ? current_algo_name : "",
-                current_mode_name ? current_mode_name : "",
-                nsecpiter_buf,
-                cpiter_buf);
+      printf ("%s,%s,%s,,,,,,,,,%s,ns/iter,%s,c/iter\n",
+	      current_section_name,
+	      current_algo_name ? current_algo_name : "",
+	      current_mode_name ? current_mode_name : "",
+	      nsecpiter_buf,
+	      cpiter_buf);
     }
   else
     {
-      if (auto_ghz)
-        printf ("%14s %13s %9s\n", nsecpiter_buf, cpiter_buf, mhz_buf);
-      else
-        printf ("%14s %13s\n", nsecpiter_buf, cpiter_buf);
+      printf ("%14s %13s\n", nsecpiter_buf, cpiter_buf);
     }
 }
 
@@ -2082,10 +1705,7 @@ kdf_bench (char **argv, int argc)
   if (!csv_mode)
     {
       printf (" %-*s | ", 24, "");
-      if (auto_ghz)
-        printf ("%14s %13s %9s\n", "nanosecs/iter", "cycles/iter", "auto Mhz");
-      else
-        printf ("%14s %13s\n", "nanosecs/iter", "cycles/iter");
+      printf ("%14s %13s\n", "nanosecs/iter", "cycles/iter");
     }
 
   if (argv && argc)
@@ -2126,8 +1746,7 @@ print_help (void)
     "",
     " options:",
     "   --cpu-mhz <mhz>           Set CPU speed for calculating cycles",
-    "                             per bytes results.  Set as \"auto\"",
-    "                             for auto-detection of CPU speed.",
+    "                             per bytes results.",
     "   --disable-hwf <features>  Disable hardware acceleration feature(s)",
     "                             for benchmarking.",
     "   --repetitions <n>         Use N repetitions (default "
@@ -2162,6 +1781,7 @@ int
 main (int argc, char **argv)
 {
   int last_argc = -1;
+  int debug = 0;
 
   if (argc)
     {
@@ -2243,15 +1863,8 @@ main (int argc, char **argv)
 	  argv++;
 	  if (argc)
 	    {
-              if (!strcmp (*argv, "auto"))
-                {
-                  auto_ghz = 1;
-                }
-              else
-                {
-                  cpu_ghz = atof (*argv);
-                  cpu_ghz /= 1000;	/* Mhz => Ghz */
-                }
+	      cpu_ghz = atof (*argv);
+	      cpu_ghz /= 1000;	/* Mhz => Ghz */
 
 	      argc--;
 	      argv++;
@@ -2278,7 +1891,7 @@ main (int argc, char **argv)
 	}
     }
 
-  xgcry_control ((GCRYCTL_SET_VERBOSITY, (int) verbose));
+  gcry_control (GCRYCTL_SET_VERBOSITY, (int) verbose);
 
   if (!gcry_check_version (GCRYPT_VERSION))
     {
@@ -2288,11 +1901,11 @@ main (int argc, char **argv)
     }
 
   if (debug)
-    xgcry_control ((GCRYCTL_SET_DEBUG_FLAGS, 1u, 0));
+    gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 1u, 0);
 
-  xgcry_control ((GCRYCTL_DISABLE_SECMEM, 0));
-  xgcry_control ((GCRYCTL_INITIALIZATION_FINISHED, 0));
-  xgcry_control ((GCRYCTL_ENABLE_QUICK_RANDOM, 0));
+  gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
+  gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 
   if (in_regression_test)
     fputs ("Note: " PGM " running in quick regression test mode.\n", stdout);
